@ECHO OFF
GOTO START


Program name:
=============
        Tic2Dir.Bat -- requires a config file (see "About the .CFG file",
                       below) and some valid TIC files.


Purpose:
--------
	Process TIC files, using 'Area' keyword matching, to move the
        associated files to defined target directories.

Parameters required:
--------------------
        %1 -------- inbound TIC file name.
        %2 -------- path to a CFG file.

        %3 -------- optional action to be taken with the TIC, when done:
                    *=> KEEP it
                    *=> KILL the TIC file
                    ++ The default is to KEEP it.

Uses these dependencies:
------------------------
	LMOD.EXE ----- another free utility by Horst Schaeffer.
	FIND.EXE ----- a supplied MS-DOS utility.
        MOVE.EXE ----- a supplied MS-DOS utility.

        XsetEvar.Bat - a temp file to SET variables extracted by LMOD.
         (Created in %TEMP%, else, the root '\'.)

Limitations:
------------
        *MOST Important* ----- Tic2Dir.Bat _can_ only handle nine (9)
                               file areas.
        ! W A R N I N G ! ---- Use beyond nine (9) areas _will_ produce
        =================      unpredictable results.

About the .CFG file:
--------------------
        *Important* ----- this file is a plain ASCII text file, consisting
                          of file area 'tags' and target area directory
                          names.

        Example: my (old) Tic2Dir.Cfg file -
     -----8<-----C-U-T--H-E-R-E----->8-----
     NODEDIFF=D:\ARCS\GENERAL
     FIDONEWS=D:\ARCS\FIDONEWS
     Z3-ELIST=D:\ARCS\GENERAL
     BFDS=D:\ARCS\DOS\UTILS
     FASTECHO=D:\ARCS\BBSUTILS
     -----8<-----C-U-T--H-E-R-E----->8-----

        The name of the CFG file is _un_important.  Be creative!  :-)

Pseudocode:
-----------
       FIND the string "Area" in the .TIC obtaining [line#1]
       Using [line#1] obtain the area 'tag' name in the .TIC
       FIND string "File" in the .TIC and get [line#2]
       Using [line#2] get the file's name in the .TIC
       FIND the tag in TIC2DIR.CFG, obtaining the target dir in TIC2DIR.CFG
       MOVE the associated file to the target directory
       Do we KEEP or KILL the incoming TIC file - default is to KEEP.


::                          ---===[ * * * ]===---
:START
  ECHO.
  ECHO  %0 [ver 2.00.g3]
  ECHO.
  ECHO  Processes TIC-associated files, using 'Area' keyword matching.
  ECHO.

  :: ********** Check the command-line info.
  :: Maybe somone wants help, first.
  for %%f in (h H ?) do if "%1"=="/%%f" goto HELP
  for %%f in (h H ?) do if "%1"=="-%%f" goto HELP
  for %%f in (h H ?) do if "%1"=="%%f" goto HELP

  :: ********** Simple checks for the required _number_ of params.
  IF [NO%2PARAM]==[NOPARAM] GOTO HOWTOUSE
  IF [NO%1PARAM]==[NOPARAM] GOTO HOWTOUSE

  :: ********** Start building a DOS environment, using the XsetEvar.Bat file.
  SET T_TEST=TICFILE
  SET T_NBR1=0123
  SET T_NBR2=0123
  SET T_TAGN=TAG
  SET T_FILE=FILENAME
  SET T_TGET=TGT_DIR
  SET T_KEEP=KEEP
  SET T_BASE=TICBAK
  
  ::   Check to make sure param #1 is in fact a TIC file.
  DIR %1 /B | LMOD /S. SET T_TEST=[$1]>%TEMP%\XsetEvar.Bat
  IF EXIST %TEMP%\XsetEvar.Bat CALL %TEMP%\XsetEvar.Bat
  IF NOT EXIST %T_TEST%.TIC GOTO HOWTOUSE

  ::   Do we have the CFG file nominated, actually in the path specified.
  IF NOT EXIST %2 GOTO HOWTOUSE

  ::   Convert the 3rd param (if there is one) to uppercase for testing.
  IF NOT [NO%3PARAM]==[NOPARAM] ECHO %3 | LMOD /S,;= SET T_KEEP=[U]>%TEMP%\XsetEvar.Bat
  IF EXIST %TEMP%\XsetEvar.Bat CALL %TEMP%\XsetEvar.Bat

  ::   FIND the string "Area" in the .TIC obtaining [line#1]
  FIND /N /I "Area" %1
  IF ERRORLEVEL 1 GOTO EL_ERR_1
  FIND /N /I "Area" %1 | LMOD /L3 SET T_NBR1=[2,1]>%TEMP%\XsetEvar.Bat
  IF ERRORLEVEL 1 GOTO EL_ERR_1
  IF EXIST %TEMP%\XsetEvar.Bat CALL %TEMP%\XsetEvar.Bat

  ::   Using [line#1] obtain the area 'tag' name in the .TIC
  LMOD /L%T_NBR1% SET T_TAGN=[$2] <%1>%TEMP%\XsetEvar.Bat
  IF ERRORLEVEL 1 GOTO EL_ERR_1
  IF EXIST %TEMP%\XsetEvar.Bat CALL %TEMP%\XsetEvar.Bat

  ::   FIND string "File" in the .TIC and get [line#2]
  FIND /N /I "File" %1
  IF ERRORLEVEL 1 GOTO EL_ERR_1
  FIND /N /I "File" %1 | LMOD /L3 SET T_NBR2=[2,1]>%TEMP%\XsetEvar.Bat
  IF ERRORLEVEL 1 GOTO EL_ERR_1
  IF EXIST %TEMP%\XsetEvar.Bat CALL %TEMP%\XsetEvar.Bat

  ::   Using [line#2] get the file's name in the .TIC
  LMOD /L%T_NBR2% SET T_FILE=[$2] <%1>%TEMP%\XsetEvar.Bat
  IF ERRORLEVEL 1 GOTO EL_ERR_1
  IF EXIST %TEMP%\XsetEvar.Bat CALL %TEMP%\XsetEvar.Bat
  IF NOT EXIST %T_FILE% GOTO ERR_MSG_1

  ::   FIND the tag in TIC2DIR.CFG, obtaining the target dir in TIC2DIR.CFG
  FIND /N /I "%T_TAGN%" %2
  IF ERRORLEVEL 1 GOTO EL_ERR_1
  FIND /N /I "%T_TAGN%" %2 | LMOD /L3 SET T_TGET=[$2]>%TEMP%\XsetEvar.Bat
  IF ERRORLEVEL 1 GOTO EL_ERR_1
  IF EXIST %TEMP%\XsetEvar.Bat CALL %TEMP%\XsetEvar.Bat
  IF NOT EXIST %T_TGET%\NUL GOTO ERR_MSG_1

  ::   MOVE the associated file to the target directory
  MOVE /Y %T_FILE% %T_TGET%

  ::   Do we KEEP or KILL the incoming TIC file - default is to KEEP.
    IF [%T_KEEP%]==[KILL] DEL %1
    IF [%T_KEEP%]==[KILL] GOTO STOPHERE
  :: Okay, then we must be KEEP-ing it.
    :: Determine a suitable filename, first, using the base filename of
    :: the TIC-attached file, renaming it to a .TAC extension.
    ECHO %T_FILE% | LMOD /S. SET T_BASE=[$1]>%TEMP%\XsetEvar.Bat
    IF EXIST %TEMP%\XsetEvar.Bat CALL %TEMP%\XsetEvar.Bat
  REN %1 %T_BASE%.TAC

  ::  End of procedure.
  GOTO STOPHERE

  :: ********** HELP!!!!!!!
  :HELP
    ECHO  %0 requires a config file and valid TIC files.  The config
    ECHO  file is a plain ASCII text file, consisting of file area 'tags'
    ECHO  and target directory paths.  (E.g., NODEDIFF=D:\ARCS\NODEDIFF)
    ECHO.
    ECHO  !WARNING! %0 can _only_ handle nine (9) file area 'tag' lines.
    ECHO.
    ECHO  Optionally KEEPs or KILLs the .TIC file when processing is done.
    ECHO.
    ECHO  The following (external) utilities _must_ be available on the DOS
    ECHO  search PATH -
    ECHO        LMOD.EXE - another free utility by Horst Schaeffer.
    ECHO        FIND.EXE - a supplied MS-DOS utility.
    ECHO        MOVE.EXE - a supplied MS-DOS utility.
    ECHO.
  GOTO HOWTOUSE

  :: *********** General-purpose error trapper for FIND/LMOD
  :EL_ERR_1
    :: Each FIND has been replicated, to trap for ErrorLevel 1 or higher,
    :: which indicates an error sufficient to require a halt.
    :: LMOD also uses an exit code (errorlevel) of 1, to indicate failure.
    :: Formally, LMOD's exit codes are:
    :: Errorlevel 0: ok, 1: invalid token/expression, 2: invalid options.
    :: Therefore, testing for [EL=>1] is enough.  (Thanks Horst!)
    Echo.
    Echo  Cannot continue; an error occurred in either FIND or LMOD.
    Echo.
    Echo  The problem TIC and its associated file will remain un-touched in
    Echo  the current directory.  %0 is now stopping...
    Echo.
  GOTO HOWTOUSE

  :: ************************** File/directory 'existence' error trap
  :ERR_MSG_1
    :: We've tested for the existence of a file or directory and DOS has
    :: come up empty-handed.
    Echo.
    Echo  Cannot find the TIC-associated file =or= the target directory!
    Echo.
  GOTO HOWTOUSE

  :: ********** Assist the unsuspecting user.
  :HOWTOUSE
    ECHO  Usage is: %0  Inbound.TIC  Path_to_.CFG_File  [KEEP/KILL]  c/r
    ECHO.
    ECHO  'Type %0 /h' for a little more help.
    ECHO.
  GOTO STOPHERE

:: ********** End of the line, baby.
:STOPHERE
  SET T_TEST=
  SET T_NBR1=
  SET T_NBR2=
  SET T_TAGN=
  SET T_FILE=
  SET T_TGET=
  SET T_KEEP=
  SET T_BASE=
IF EXIST %TEMP%\XsetEvar.Bat DEL %TEMP%\XsetEvar.Bat


::                          ---===[ * * * ]===---
:: Tic2Dir is free!  Enjoy!  =8-)  (c)1999 Paul D. Quinn
::   FidoNet: 3:640/384.0, e-mail: eskimo261@bigfoot.com -- 5 October 2000.

:: Many thanks to LMOD's author, Horst Schaeffer, for another beaut utility!